diff mbox

Avoid .symver on common symbols [BZ #21666]

Message ID 20170623161158.GA5384@gmail.com
State New
Headers show

Commit Message

H.J. Lu June 23, 2017, 4:11 p.m. UTC
The .symver directive on common symbol just creates a new common symbol,
not an alias and the newer assembler with the bug fix for

https://sourceware.org/bugzilla/show_bug.cgi?id=21661

will issue an error.  Before the fix, we got

$ readelf -sW libc.so | grep "loc[12s]"
  5109: 00000000003a0608     8 OBJECT  LOCAL  DEFAULT   36 loc1
  5188: 00000000003a0610     8 OBJECT  LOCAL  DEFAULT   36 loc2
  5455: 00000000003a0618     8 OBJECT  LOCAL  DEFAULT   36 locs
  6575: 00000000003a05f0     8 OBJECT  GLOBAL DEFAULT   36 locs@GLIBC_2.2.5
  7156: 00000000003a05f8     8 OBJECT  GLOBAL DEFAULT   36 loc1@GLIBC_2.2.5
  7312: 00000000003a0600     8 OBJECT  GLOBAL DEFAULT   36 loc2@GLIBC_2.2.5

in libc.so.  The versioned loc1, loc2 and locs have the wrong addresses.
After the fix, we got

$ readelf -sW libc.so | grep "loc[12s]"
  6570: 000000000039e3b8     8 OBJECT  GLOBAL DEFAULT   34 locs@GLIBC_2.2.5
  7151: 000000000039e3c8     8 OBJECT  GLOBAL DEFAULT   34 loc1@GLIBC_2.2.5
  7307: 000000000039e3c0     8 OBJECT  GLOBAL DEFAULT   34 loc2@GLIBC_2.2.5

OK for master?

Thanks.

H.J.
---
	[BZ #21666]
	* misc/regexp.c: Include <stdlib.h>.
	(loc1): Initialized to NULL.
	(loc2): Likewise.
	(locs): Likewise.
---
 misc/regexp.c | 11 +++++++----
 1 file changed, 7 insertions(+), 4 deletions(-)

Comments

Florian Weimer June 23, 2017, 4:17 p.m. UTC | #1
On 06/23/2017 06:11 PM, H.J. Lu wrote:
> +/* Define the variables used for the interface.  Avoid .symver on common
> +   symbol, which just creates a new common symbol, not an alias.  */
> +char *loc1 = NULL;
> +char *loc2 = NULL;

I think __attribute__ ((nocommon)) without the initializer would be more
explicit.  We already use that for _res in resolv/res_libc.c.

Does this result in a visible difference for applications?  If yes,
please file a bug for this and reference it in the ChangeLog and commit
message.

Why didn't our test suite catch it?

Thanks,
Florian
diff mbox

Patch

diff --git a/misc/regexp.c b/misc/regexp.c
index 19d76c0..9017bc1 100644
--- a/misc/regexp.c
+++ b/misc/regexp.c
@@ -29,14 +29,17 @@ 
 
 #if SHLIB_COMPAT (libc, GLIBC_2_0, GLIBC_2_23)
 
-/* Define the variables used for the interface.  */
-char *loc1;
-char *loc2;
+#include <stdlib.h>	/* Get NULL.  */
+
+/* Define the variables used for the interface.  Avoid .symver on common
+   symbol, which just creates a new common symbol, not an alias.  */
+char *loc1 = NULL;
+char *loc2 = NULL;
 compat_symbol (libc, loc1, loc1, GLIBC_2_0);
 compat_symbol (libc, loc2, loc2, GLIBC_2_0);
 
 /* Although we do not support the use we define this variable as well.  */
-char *locs;
+char *locs = NULL;
 compat_symbol (libc, locs, locs, GLIBC_2_0);